Arabic_Words = Core.class(Sprite)

function Arabic_Words:init(
			pl01, pl02, pl03, pl04, pl05, pl06, pl07, pl08, pl09, pl10, pl11, pl12, pl13, pl14, pl15,
			pl01f, pl02f, pl03f, pl04f, pl05f, pl06f, pl07f, pl08f, pl09f, pl10f, pl11f, pl12f, pl13f, pl14f, pl15f,
			pv01, pv02, pv03, pv04, pv05, pv06, pv07, pv08, pv09, pv10, pv11, pv12, pv13, pv14, pv15)

	-- set up the letters tilesheet, 1 letter is 64*64px
	local texture = Texture.new("arabic/arabletters_64_dmf.png", false)

	-- fills the table with each letter texture (i, d, m, f)
	self.arabletters_list = {}
	for l = 1, 8 do -- number of lines in tilesheet
		for c = 1, 15 do -- number of columns in tilesheet
			local arabletter = TextureRegion.new(texture, (c-1) * 64, (l-1) * 64, 64, 64)
			local bitmap = Bitmap.new(arabletter)
			self.arabletters_list[#self.arabletters_list + 1] = bitmap
		end
	end
	
	-- picks the right letter form depending on its position in the word, and fills a table
	local xlf = {
		self:letterForm(pl01, pl01f),
		self:letterForm(pl02, pl02f),
		self:letterForm(pl03, pl03f),
		self:letterForm(pl04, pl04f),
		self:letterForm(pl05, pl05f),
		self:letterForm(pl06, pl06f),
		self:letterForm(pl07, pl07f),
		self:letterForm(pl08, pl08f),
		self:letterForm(pl09, pl09f),
		self:letterForm(pl10, pl10f),
		self:letterForm(pl11, pl11f),
		self:letterForm(pl12, pl12f),
		self:letterForm(pl13, pl13f),
		self:letterForm(pl14, pl14f),
		self:letterForm(pl15, pl15f),
	}

	-- position each letters 32px apart from the other (it's RTL)
	for lposx = 1, #xlf do
		if lposx == 1 then
			-- there is no letter before the first one so we pass it for the letter minus 1
			self:letterPosX(lposx, xlf[lposx], xlf[lposx - 0])
		else
			self:letterPosX(lposx, xlf[lposx], xlf[lposx - 1])
		end
	end

	-- assigns a vowel from the function paramaters
	local xlv = {
		self:letterVowel(xlf[1], pv01),
		self:letterVowel(xlf[2], pv02),
		self:letterVowel(xlf[3], pv03),
		self:letterVowel(xlf[4], pv04),
		self:letterVowel(xlf[5], pv05),
		self:letterVowel(xlf[6], pv06),
		self:letterVowel(xlf[7], pv07),
		self:letterVowel(xlf[8], pv08),
		self:letterVowel(xlf[9], pv09),
		self:letterVowel(xlf[10], pv10),
		self:letterVowel(xlf[11], pv11),
		self:letterVowel(xlf[12], pv12),
		self:letterVowel(xlf[13], pv13),
		self:letterVowel(xlf[14], pv14),
		self:letterVowel(xlf[15], pv15),
	}

	-- finally adds the letters and its vowel to self
	for xlfv = 1, #xlf do
		self:addChild(xlf[xlfv])
		self:addChild(xlv[xlfv])
	end
	
end

-- FUNCTIONS
function Arabic_Words:letterForm(pl, plf)

	if pl ~= nil then

		local xlf = nil -- holder for the correct texture

		if pl == 0 then
			-- if the letter is 0 then this is a space (texture = 1)
			xlf = self.arabletters_list[1]

		else
			-- we pick the correct letter form from the texture pack
			if plf == "i" then
				xlf = self.arabletters_list[pl * 4 - 2]
			elseif plf == "d" then
				xlf = self.arabletters_list[pl * 4 - 1]
			elseif plf == "m" then
				xlf = self.arabletters_list[pl * 4 - 0]
			elseif plf == "f" then
				xlf = self.arabletters_list[pl * 4 + 1]
			elseif plf == "md" then
				xlf = self.arabletters_list[pl * 4 - 1]
			elseif plf == "mf" then
				xlf = self.arabletters_list[pl * 4 + 1]
			elseif plf == "fi" then
				xlf = self.arabletters_list[pl * 4 - 2]
			else
				-- nothing here
			end
	
		end

		return xlf
	
	end
	
	return nil

end

function Arabic_Words:letterPosX(pletterindex, pletter, pletterminus1)

	if pletterindex > 1 then -- position the letters based on the first letter x position
	
		if pletterminus1 == self.arabletters_list[1] then -- this is a space 

			pletter:setX(pletterminus1:getX() - 8)

		else -- this is a normal letter

			pletter:setX(pletterminus1:getX() - 32)
	
		end

	end

end

function Arabic_Words:letterVowel(pl, pv)

	if pl ~= nil then

		local xlv = nil

		if pv == "a" then
			xlv = Bitmap.new(Texture.new("arabic/a_i.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 16)
		elseif pv == "ou" then
			xlv = Bitmap.new(Texture.new("arabic/ou.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 16)
		elseif pv == "i" then
			xlv = Bitmap.new(Texture.new("arabic/a_i.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() + 48)
		elseif pv == "s" then
			xlv = Bitmap.new(Texture.new("arabic/souk.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 16)
		elseif pv == "oun" then
			xlv = Bitmap.new(Texture.new("arabic/oun.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 16)
		elseif pv == "ca" then
			xlv = Bitmap.new(Texture.new("arabic/chadda.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 16)
		elseif pv == "as" then
			xlv = Bitmap.new(Texture.new("arabic/alifspec.png"))
			xlv:setPosition(pl:getX() + 16, pl:getY() - 0)
		elseif pv == " " then
			xlv = Bitmap.new(Texture.new("arabic/empty.png"))
		elseif pv == "." then
			xlv = Bitmap.new(Texture.new("arabic/point.png"))
			xlv:setPosition(pl:getX() + 32, pl:getY() + 24)
		elseif pv == ":" then
			xlv = Bitmap.new(Texture.new("arabic/deux_points.png"))
			xlv:setPosition(pl:getX() + 32, pl:getY() + 20)
		else
			xlv = Bitmap.new(Texture.new("arabic/empty.png"))
		end

		if pv == "." or pv == ":" then
			xlv:setScale(0.35, 0.35)
		else
			xlv:setScale(0.25, 0.25)
		end

		return xlv

	end
	
	return nil

end

