-- GLOBALS
--myappwidth = application:getDeviceWidth()
--myappheight = application:getDeviceHeight()
myappwidth = application:getContentWidth()
myappheight = application:getContentHeight()

function math.dist(x1, y1, x2, y2) return ((x2-x1)^2+(y2-y1)^2)^0.5 end
function math.angle(x1, y1, x2, y2) return math.atan2(y2 - y1, x2 - x1) end

function isColliding(a1, a2) -- two bitmap objects
	if(a1 == a2) then return false end
	local dx = a1.posx - a2.posx
	local dy = a1.posy - a2.posy
	if math.abs(dx) < a1.bmp:getWidth() / 2 + a2.bmp:getWidth() / 2 then
		if math.abs(dy) < a1.bmp:getHeight() / 2 + a2.bmp:getHeight() / 2 then
			return true
		end
	end
	
	return false
end

function isColliding2(a1, a2) -- two sprite objects
	if(a1 == a2) then return false end
	local dx = a1.posx - a2.posx
	local dy = a1.posy - a2.posy
	if math.abs(dx) < a1.w / 2 + a2.w / 2 then
		if math.abs(dy) < a1.h / 2 + a2.h / 2 then
			return true
		end
	end
	
	return false
end

-- GLOBALS

-- HELPERS CLASS
-- Arabic
-- Button
-- xCircle
-- xCenteredTextField

-- ******************************************************************************

-- ******************************************************************************

function arabicProcessing(text)
	local Achar={0x002,0x024,0x062,0x084,0x0C4,0x104,0x144,0x184,0x1C2,0x1E2,0x202,0x222,
		0x244,0x284,0x2C4,0x304,0x344,0x384,0x3C4,0x404,0x440,0x440,0x440,0x440,0x440,0x440,
		0x444,0x484,0x4C4,0x504,0x544,0x584,0x5C4,0x602,0x622,0x644}

	local last=0	
	local rtext=""

	for p, cur in utf8.codes(text) do 

		if ((last >= 0x627) and (last <= 0x64A)) then
			local A = Achar[last-0x626]
			local unibase = 0xFE8D + (A>>4)
			local of = 0 --0=iso,1=fin,2=ini,3=med

			if ((cur >= 0x627) and (cur <= 0x64A)) then
				of = 2 + state
			else
				of = state
			end

			state = 1

			if of >= A&0xF then of = 0 end

			of = of + unibase
			rtext = rtext..utf8.char(of)

		else
			if last > 0 then
				rtext = rtext..utf8.char(last)
			end
			state = 0
		end

		last = cur

	end

	if last > 0 then
		rtext = rtext..utf8.char(last)
	end

	text = ""
	for p, cur in utf8.codes(rtext) do
		if cur == 0x28 or cur == 0x29 then cur = cur ~ 1 end -- this swaps brackets
		text = utf8.char(cur)..text
	end

	return text

end

-- ******************************************************************************

-- ******************************************************************************

function loadCSV(file)

	local function csvsplit(line)
		local t,w,l2= {}, nil, nil

		while #line > 0 do
			w, l2 = line:match("\"([^\"]*)\"[,\r\n]?(.*)") -- Check for quoted string
			if not w then w, line = line:match("([^,]*)[,\r\n]?(.*)") -- Non quoted 
			else line = l2 end
			if not w then break end --Nothing or issue, break
			t[#t+1] = w
		end
		return t
	end

	local header,csv = nil, {}
	for line in io.lines(file) do
		f = csvsplit(line)
		if not headers then -- Assume first line is headers
			headers = {} for n, v in ipairs(f) do headers[v] = n end
		else
			csv[#csv+1] = f
		end
	end

	csv.getField = function (self,row,field) return self[row][headers[field]] end

	return csv

end
 
--csv=loadCSV("135-bis-bt.csv")
--print(csv:getField(4,"Value "))
--print(csv:getField(7,"Time")
 
-- ******************************************************************************

-- ******************************************************************************

Button = Core.class(Sprite)

function Button:init(text, tfont, tcolor, tscalex, tscaley, upState, downState)
	self.text = TextField.new(tfont, text) -- string
	
	if upState ~= nil and downState ~= nil then
		self.upState = Bitmap.new(Texture.new(upState))-- bmp
		self.downState = Bitmap.new(Texture.new(downState)) -- bmp
		self.bmpwidth = self.upState:getWidth()
		self.bmpheight = self.upState:getHeight()
		self.hasbmp = true
	elseif self.text ~= nil then
		self.bmpwidth = self.text:getWidth()
		self.bmpheight = self.text:getHeight()
		self.hasbmp = false
	else
		print("nothing on the button?, strange!")
	end


	self.focus = false

	self:mySetText(tcolor, tscalex, tscaley)
	
	if self.hasbmp then
		self:updateVisualState(false)
	end

	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

-- BUTTON FUNCTIONS
function Button:mySetText(pcolor, pscalex, pscaley)
	self.text:setAnchorPoint(0.5, 0.5)
	self.text:setPosition(0, self.bmpheight / 2)
	self.text:setTextColor(pcolor)
	self.text:setScale(pscalex, pscaley)
	self:addChild(self.text)
end

function Button:updateVisualState(state)
	if self.hasbmp then
		self.upState:setAnchorPoint(0.5, 0.5)
		self.downState:setAnchorPoint(0.5, 0.5)
		-- if state is true show downState bitmap else show upState bitmap
		if state then
			if self:contains(self.upState) then
				self:removeChild(self.upState)
			end
			
			if not self:contains(self.downState) then
				self:addChild(self.downState)
			end
		else
			if self:contains(self.downState) then
				self:removeChild(self.downState)
			end
			
			if not self:contains(self.upState) then
				self:addChild(self.upState)
			end
		end

		self:mySetText() -- need to call here otherwise the text is going behind the button
	end
end

-- BUTTON LISTENERS
function Button:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true
		self:updateVisualState(true)
		event:stopPropagation()
	end
end

function Button:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then
			self.focus = false;
			self:updateVisualState(false)
		end
		event:stopPropagation()
	end
end

function Button:onMouseUp(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		self:dispatchEvent(Event.new("click"))
		event:stopPropagation()
	end
end

function Button:onTouchesBegin(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function Button:onTouchesMove(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function Button:onTouchesEnd(event)
	-- if button is on focus, stop propagation of touch events
	if self.focus then
		event:stopPropagation()
	end
end

function Button:onTouchesCancel(event)
	-- if touches are cancelled, reset the state of the button
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		event:stopPropagation()
	end
end

-- ******************************************************************************

-- ******************************************************************************

xCircle = Core.class(Sprite)

function xCircle:init(r, steps, color)
	color = color or 0xff0000

	local sin, cos, d2r = math.sin, math.cos, math.pi / 180

	local c = Shape.new()
	c:setFillStyle(Shape.SOLID, color)
	c:setLineStyle(0)
	c:beginPath()
	c:moveTo(r * sin(0 * d2r), r * cos(0 * d2r))
	for i = 0, 360, 360 / steps  do
	  c:lineTo(r * sin(i * d2r), r * cos(i * d2r))
	end    
	c:endPath()

	self:addChild(c)
end

-- ******************************************************************************

-- ******************************************************************************

xCenteredTextField = Core.class(Sprite)

function xCenteredTextField:init(ptext, pfont, pcolor, pscalex, pscaley, pposx, pposy)
	local mytextfield = TextField.new(pfont, ptext)
	mytextfield:setTextColor(pcolor or 0x00ff00)
	mytextfield:setScale(pscalex, pscaley)
	mytextfield:setAnchorPoint(0.5, 0.5)
	mytextfield:setPosition(pposx, pposy)
	self:addChild(mytextfield)
end

-- ******************************************************************************

-- ******************************************************************************






