TILEW @ 32
TILEH @ 32

Level01 = Core.class(Sprite)

function Level01:init()

	-- level 01 variables
	self.listSprites = {}
	self.map = {}
	self.tiles = {}

	-- functions
	self:loadLevel()
	
	-- text reference
	local label = TextField.new(nil, "TEST")
	label:setTextColor(0xffffff)
	label:setScale(2, 2)
	label:setPosition(0, label:getHeight() * label:getScaleY())
	self:addChild(label)

	-- LISTENERS
	self:addEventListener("enterBegin", self.onTransitionInBegin, self)
	self:addEventListener("enterEnd", self.onTransitionInEnd, self)
	self:addEventListener("exitBegin", self.onTransitionOutBegin, self)
	self:addEventListener("exitEnd", self.onTransitionOutEnd, self)

end

function Level01:loadLevel()

	-- fills the map table with the map file
	local filename = "maps/map1.txt"
	for line in io.lines(filename) do
		self.map[#self.map + 1] = line
	end

	-- the tiles
	local texture = Texture.new("gfx/tiles/32_aplevel01.png")
	self.tiles["1"] = TextureRegion.new(texture, 0, 0, 32, 32)
	self.tiles["2"] = TextureRegion.new(texture, 32, 0, 32, 32)
	self.tiles["3"] = TextureRegion.new(texture, 64, 0, 32, 32)

	-- draw map
	for l = 1, #self.map do
		for c = 1, #self.map[l] - 1 do
			local char = string.sub(self.map[l], c, c)
			if tonumber(char) > 0 then
				local tile = Bitmap.new(self.tiles[char])
				tile:setPosition((c - 1) * TILEW, (l - 1) * TILEH)
				self:addChild(tile)
			end
		end
	end

	-- hero
	self.hero = Hero.new(4 * TILEW, 7 * TILEH)
	self:addChild(self.hero)
	table.insert(self.listSprites, self.hero)
	
end

-- functions
function Level01:getTileAt(pX, pY)
	local col = math.floor(pX / TILEW) + 1
	local lig = math.floor(pY / TILEH) + 1

	if col > 0 and col <= #self.map[1] and lig > 0 and lig <= #self.map then
		local id = string.sub(self.map[lig], col, col)
--		print(id)
		return id
	end

	return 0
end

-- GAME LOOP
function Level01:onEnterFrame(e)
	for s = #self.listSprites, 1, -1 do
		local sprite = self.listSprites[s]
		self:updateSprite(sprite, e.deltaTime)
	end
end

function Level01:updatePlayer(psprite, dt)
	local accel = 500
	local friction = 150
	local maxspeed = 150
	local jumpspeed = -280

	-- Friction
	if psprite.vx > 0 then
		psprite.vx = psprite.vx - friction * dt
		if psprite.vx < 0 then psprite.vx = 0 end
	end

	if psprite.vx < 0 then
		psprite.vx = psprite.vx + friction * dt
		if psprite.vx > 0 then psprite.vx = 0 end
	end

	-- Keyboard
	if psprite.iskeyright then
		psprite.vx = psprite.vx + accel * dt
		if psprite.vx > maxspeed then psprite.vx = maxspeed end
		psprite.flip = false
	end

	if psprite.iskeyleft then
		psprite.vx = psprite.vx - accel * dt
		if psprite.vx < -maxspeed then psprite.vx = -maxspeed end
		psprite.flip = true
	end

	if psprite.iskeyup and psprite.isstanding and psprite.isjumpready then
		psprite.vy = jumpspeed
		psprite.isstanding = false
		psprite.isjumpready = false
	end

	if psprite.iskeyup == false and psprite.isjumpready == false then
		psprite.isjumpready = true
	end

	local flipCoef = 1
	if psprite.flip then flipCoef = -1 end

	-- Move & scale
	psprite.x = psprite.x + psprite.vx * dt
	psprite.y = psprite.y + psprite.vy * dt
	
	psprite:setPosition(psprite.x, psprite.y)
	psprite:setScale(flipCoef, 1)

end

function Level01:isSolid(pID)
	if pID == "0" then return false end
	if pID == "1" then return true end
	if pID == "2" then return true end
end

function Level01:isJumpThrough(pid)
	if pid == "3" then return true end
	return false
end

function Level01:CollideRight(pSprite)
	local id1 = self:getTileAt(pSprite.x + pSprite:getWidth() + 1, pSprite.y + 3)
	local id2 = self:getTileAt(pSprite.x + pSprite:getWidth() + 1, pSprite.y + pSprite:getHeight() - 3)
	if self:isSolid(id1) or self:isSolid(id2) then return true end
	return false
end

function Level01:CollideLeft(pSprite)
	local id1 = self:getTileAt(pSprite.x - 1, pSprite.y + 3)
	local id2 = self:getTileAt(pSprite.x - 1, pSprite.y + pSprite:getHeight() - 3)
	if self:isSolid(id1) or self:isSolid(id2) then return true end
	return false
end

function Level01:CollideBelow(pSprite)
	local id1 = self:getTileAt(pSprite.x + 3, pSprite.y + pSprite:getHeight() + 1)
	local id2 = self:getTileAt(pSprite.x + pSprite:getWidth() - 3, pSprite.y + pSprite:getHeight() + 1)
	if self:isSolid(id1) or self:isSolid(id2) then return true end

	if self:isJumpThrough(id1) or self:isJumpThrough(id2) then
		local lig = math.floor((pSprite.y + pSprite:getHeight() / 2) / pSprite:getHeight()) + 1
		local yLine = (lig - 1) * 32
		local distance = pSprite.y - yLine
		if distance >= 0 and distance < 10 then
			return true
		end
	end

	return false
end

function Level01:CollideAbove(pSprite)
	local id1 = self:getTileAt(pSprite.x + 3, pSprite.y - 1)
	local id2 = self:getTileAt(pSprite.x + pSprite:getWidth() - 3, pSprite.y - 1)
	if self:isSolid(id1) or self:isSolid(id2) then return true end
	return false
end

function Level01:updateSprite(pSprite, dt)

	-- Locals for Collisions
	local oldX = pSprite.x
	local oldY = pSprite.y

	-- Specific behavior for the player
	self:updatePlayer(pSprite, dt)
	-- Collision detection
	local collide = false
	-- On the right
	if pSprite.vx > 0 then collide = self:CollideRight(pSprite) end
	-- On the left
	if pSprite.vx < 0 then collide = self:CollideLeft(pSprite) end
	-- Stop!
	if collide then 
		pSprite.vx = 0
		local col = math.floor((pSprite.x + TILEW / 2) / TILEW) + 1
		pSprite.x = (col - 1) * TILEW
	end

	collide = false
	-- Above
	if pSprite.vy < 0 then
		collide = self:CollideAbove(pSprite)
		if collide then
			pSprite.vy = 0
			local lig = math.floor((pSprite.y + TILEW / 2) / TILEW) + 1
			pSprite.y = (lig - 1) * TILEW
		end
	end

	collide = false
	-- Below
	if pSprite.isstanding or pSprite.vy > 0 then
		collide = self:CollideBelow(pSprite)
		if collide then
			pSprite.isstanding = true
			pSprite.vy = 0
			local lig = math.floor((pSprite.y + TILEH / 2) / TILEH) + 1
			pSprite.y = (lig - 1) * TILEH
		else
			pSprite.isstanding = false
		end
	end

	-- Sprite falling
	if pSprite.isstanding == false then
		pSprite.vy = pSprite.vy + 550 * dt
	end

end

-- EVENT LISTENERS
function Level01:onTransitionInBegin()
end

function Level01:onTransitionInEnd()
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self:myKeysPressed()
end

function Level01:onTransitionOutBegin()
	self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Level01:onTransitionOutEnd()
end

-- KEYS HANDLER
function Level01:myKeysPressed()
	self:addEventListener(Event.KEY_DOWN, function(e)
		if e.keyCode == KeyCode.BACK or e.keyCode == KeyCode.ESC then
			local transition = transitions[2]
			sceneManager:changeScene("menu", 1, transition, easing.outBack)
		end
	end)
end


